@@ -52,3 +52,16 @@ pub use portability::{
 
 pub mod example_ffi;
 pub mod views;
+
+// Ideally, we'd want crates to implement our traits themselves. But for now,
+// while we're prototyping, we provide a few impls on foreign types.
+#[cfg(feature = "async-std")]
+mod impls_async_std;
+#[cfg(feature = "mio")]
+mod impls_mio;
+#[cfg(feature = "os_pipe")]
+mod impls_os_pipe;
+#[cfg(feature = "socket2")]
+mod impls_socket2;
+#[cfg(feature = "tokio")]
+mod impls_tokio;
